<%-- $Id: share.jsp 5977 2012-02-20 13:26:26Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	Configure the order and visibility of columns in a table of items.
	This page will use the TableColumn.getTableColumns(item) to retreive the
	list of all available columns for that item type and
	getSession().getSetting(item+".columns") to retreive a comma-separated
	list of the current order of visible columns.

	@param item The type of items in the table

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemKey"
	import="net.sf.basedb.core.Nameable"
	import="net.sf.basedb.core.SharedItem"
	import="net.sf.basedb.core.ProjectKey"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.MultiPermissions"
	import="net.sf.basedb.core.User"
	import="net.sf.basedb.core.Group"
	import="net.sf.basedb.core.Project"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultIterator"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.PermissionUtil"
	import="net.sf.basedb.clients.web.WebException"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="java.util.EnumSet"
	import="java.util.Set"
	import="java.util.Map"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%!
private static final int READ_CODE = PermissionUtil.getPermissionCode(EnumSet.of(Permission.READ));
private static final int USE_CODE = PermissionUtil.getPermissionCode(EnumSet.of(Permission.USE));
private static final int WRITE_CODE = PermissionUtil.getPermissionCode(EnumSet.of(Permission.WRITE));
private static final int DELETE_CODE = PermissionUtil.getPermissionCode(EnumSet.of(Permission.DELETE));
private static final int SET_OWNER_CODE = PermissionUtil.getPermissionCode(EnumSet.of(Permission.SET_OWNER));
private static final int SET_PERMISSION_CODE = PermissionUtil.getPermissionCode(EnumSet.of(Permission.SET_PERMISSION));

private String getPermissionString(Set<Permission> permissions)
{
	StringBuilder sb = new StringBuilder();
	sb.append(permissions.contains(Permission.READ) ? "R" : "-");
	sb.append(permissions.contains(Permission.USE) ? "U" : "-");
	sb.append(permissions.contains(Permission.WRITE) ? "W" : "-");
	sb.append(permissions.contains(Permission.DELETE) ? "D" : "-");
	sb.append(permissions.contains(Permission.SET_OWNER) ? "O" : "-");
	sb.append(permissions.contains(Permission.SET_PERMISSION) ? "P" : "-");
	return sb.toString();
}
%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final Item itemType = Item.valueOf(request.getParameter("item_type"));
final String subContext = Values.getString(request.getParameter("subcontext"), "");

final DbControl dc = sc.newDbControl();
ItemResultIterator<User> users = null;
ItemResultIterator<Group> groups = null;
ItemResultIterator<Project> projects = null;
try
{
	MultiPermissions mp = (MultiPermissions)sc.getCurrentContext(itemType, subContext).getObject("MultiPermissions");
	if (mp == null)
	{
		throw new WebException("popup", "Item not found", "No MultiPermissions object found for item: " + itemType);
	}

	// Query to retrieve users
	final ItemQuery<User> userQuery = mp.getUsers();
	userQuery.order(Orders.asc(Hql.property("name")));

	// Query to retrieve groups
	final ItemQuery<Group> groupQuery = mp.getGroups();
	groupQuery.order(Orders.asc(Hql.property("name")));

	// Query to retrieve projects
	final ItemQuery<Project> projectQuery = mp.getProjects();
	projectQuery.order(Orders.asc(Hql.property("name")));
	
	final boolean writePermission = true;
	
	/*
	 	Set the headline with the item's name or, 
	 	if there are more then one item, the text 'multiple item'
	*/
	String headLine = "";
	Set<SharedItem> currentItems = mp.getItems();
	if(currentItems.size() == 1)
	{
		SharedItem item = currentItems.iterator().next();  
		if(item instanceof Nameable)
		{
			headLine = "Set access permissions -- " + HTML.encodeTags(((Nameable)item).getName());
		}
		else
		{
			headLine = "Set access permissions -- " + HTML.encodeTags(item.toString());
		}
	}
	else
	{
		headLine = "Set access permissions -- Multiple items (" + currentItems.size() + ")";
	}
	boolean hasDirectory = mp.getItems(Item.DIRECTORY).size() > 0;

	%>
	<base:page type="popup" title="Set access permissions">
	<base:head scripts="linkitems.js" >
		<script language="JavaScript">
		var share_to_everyone = <%=sc.hasSystemPermission(Permission.SHARE_TO_EVERYONE)%>;
		var everyone_id = <%=SystemItems.getId(Group.EVERYONE)%>;
		function initList()
		{
			var members = document.forms['share'].members;
			Link.addNewSection(members, new Section('U', 'Users'));
			<%
			users = userQuery.iterate(dc);
			while (users.hasNext())
			{
				User user = users.next();
				Map<ItemKey, Set<Permission>> p = mp.getPermissions(user);
				String permissionString = "varying";
				int permissionCode = 0;
				if (MultiPermissions.allSame(p.values()))
				{
					Set<Permission> first = p.values().iterator().next();
					permissionString = getPermissionString(first);
					permissionCode = PermissionUtil.getPermissionCode(first);
				}
				%>
				Link.addNewItem(members, new Item('U', <%=user.getId()%>, '<%=HTML.javaScriptEncode(user.getName())%> [<%=permissionString%>]', <%=permissionCode%>));
				<%
			}
			%>
			Link.addNewSection(members, new Section('G', 'Groups'));
			<%
			groups = groupQuery.iterate(dc);
			while (groups.hasNext())
			{
				Group group = groups.next();
				Map<ItemKey, Set<Permission>> p = mp.getPermissions(group);
				String permissionString = "varying";
				int permissionCode = 0;
				if (MultiPermissions.allSame(p.values()))
				{
					Set<Permission> first = p.values().iterator().next();
					permissionString = getPermissionString(first);
					permissionCode = PermissionUtil.getPermissionCode(first);
				}
				%>
				Link.addNewItem(members, new Item('G', <%=group.getId()%>, '<%=HTML.javaScriptEncode(group.getName())%> [<%=permissionString%>]', <%=permissionCode%>));
				<%
			}
			%>
			Link.addNewSection(members, new Section('P', 'Projects'));
			<%
			projects = projectQuery.iterate(dc);
			while (projects.hasNext())
			{
				Project project = projects.next();
				Map<ProjectKey, Set<Permission>> p = mp.getPermissions(project);
				String permissionString = "varying";
				int permissionCode = 0;
				if (MultiPermissions.allSame(p.values()))
				{
					Set<Permission> first = p.values().iterator().next();
					permissionString = getPermissionString(first);
					permissionCode = PermissionUtil.getPermissionCode(first);
				}
				%>
				Link.addNewItem(members, new Item('P', <%=project.getId()%>, '<%=HTML.javaScriptEncode(project.getName())%> [<%=permissionString%>]', <%=permissionCode%>));
				<%
			}
			%>
			Link.addNewSection(members, new Section('T', 'Permission templates'));
		}
		// Submit the form
		function saveSettings()
		{
			var frm = document.forms['share'];
			frm.modifiedUsers.value = Link.exportModified(frm, 'U').join(',');
			frm.modifiedGroups.value = Link.exportModified(frm, 'G').join(',');
			frm.modifiedProjects.value = Link.exportModified(frm, 'P').join(',');
			frm.permissionTemplates.value = Link.getListIds(frm.members, 'T').join(',');
			frm.submit();
		}
		function correctPermissions(checked)
		{
			var frm = document.forms['share'];
			if (checked)
			{
				frm['set_permission'].checked = frm['set_permission'].checked;
				frm['set_owner'].checked = frm['set_owner'].checked;
				frm['delete'].checked = frm['delete'].checked ;
				frm['write'].checked = frm['write'].checked || frm['delete'].checked || frm['set_owner'].checked || frm['set_permission'].checked;
				frm['use'].checked = frm['use'].checked || frm['write'].checked;
				frm['read'].checked = frm['read'].checked || frm['use'].checked;
			}
			else
			{
				frm['read'].checked = frm['read'].checked;
				frm['use'].checked = frm['use'].checked && frm['read'].checked;
				frm['write'].checked = frm['write'].checked && frm['use'].checked;
				frm['delete'].checked = frm['delete'].checked && frm['write'].checked;
				frm['set_owner'].checked = frm['set_owner'].checked && frm['write'].checked;
				frm['set_permission'].checked = frm['set_permission'].checked && frm['write'].checked;
			}
		}
		function getPermissionCode()
		{
			var permissionCode = 0;
			var frm = document.forms['share'];
			permissionCode += frm['read'].checked ? <%=READ_CODE%>: 0;
			permissionCode += frm['use'].checked ? <%=USE_CODE%>: 0;
			permissionCode += frm['write'].checked ? <%=WRITE_CODE%>: 0;
			permissionCode += frm['delete'].checked ? <%=DELETE_CODE%>: 0;
			permissionCode += frm['set_owner'].checked ? <%=SET_OWNER_CODE%>: 0;
			permissionCode += frm['set_permission'].checked ? <%=SET_PERMISSION_CODE%>: 0;
			return permissionCode;
		}
		function getPermissionString(permissionCode)
		{
			var s = '';
			s += permissionCode & <%=READ_CODE%> ? "R" : "-";
			s += permissionCode & <%=USE_CODE%> ? "U" : "-";
			s += permissionCode & <%=WRITE_CODE%> ? "W" : "-";
			s += permissionCode & <%=DELETE_CODE%> ? "D" : "-";
			s += permissionCode & <%=SET_OWNER_CODE%> ? "O" : "-";
			s += permissionCode & <%=SET_PERMISSION_CODE%> ? "P" : "-";
			return s;
		}
		function showPermissions(permissionCode, isEveryone, isTemplate)
		{
			var disabled = isTemplate || (isEveryone && !share_to_everyone);
			var frm = document.forms['share'];
			frm['read'].checked = permissionCode & <%=READ_CODE%>;
			frm['use'].checked = permissionCode & <%=USE_CODE%>;
			frm['write'].checked = permissionCode & <%=WRITE_CODE%>;
			frm['delete'].checked = permissionCode & <%=DELETE_CODE%>;
			frm['set_owner'].checked = permissionCode & <%=SET_OWNER_CODE%>;
			frm['set_permission'].checked = permissionCode & <%=SET_PERMISSION_CODE%>;
			frm['read'].disabled = disabled;
			frm['use'].disabled = disabled;
			frm['write'].disabled = disabled;
			frm['delete'].disabled = disabled;
			frm['set_owner'].disabled = disabled;
			frm['set_permission'].disabled = disabled;
			if (isEveryone && !share_to_everyone)
			{
				Main.show('share_disabled');
			}
			else
			{
				Main.hide('share_disabled');
			}
		}
		
		function permissionsOnClick(name)
		{
			var frm = document.forms['share'];
			var radio = frm[name];
			correctPermissions(radio.checked);
			var permissionCode = getPermissionCode();
			var permissionString = getPermissionString(permissionCode);
			for (var i = 0; i < frm.members.length; i++)	// >
			{
				var option = frm.members[i];
				if (option.selected && option.item && option.item.id)
				{
					var item = option.item;
					var disabled = item.type == 'T' || (item.id == everyone_id && item.type == 'G' && !share_to_everyone);
					if (!disabled)
					{
						option.item.value = permissionCode;
						var text = option.text.replace(/\[.*\]/, '['+permissionString+']');
						option.text = text; 
					}
				}
			}
		}
	
		function membersOnChange()
		{
			var frm = document.forms['share'];
			var item = frm.members[frm.members.selectedIndex].item;
			if (item && item.id)
			{
				var isTemplate = item.type == 'T';
				var isEveryone = item.id == everyone_id && item.type == 'G';
				showPermissions(item.value, isEveryone, isTemplate);
			}
		}

		function addUsersOnClick()
		{
			var ids = Link.getListIds(document.forms['share'].members, 'U');
			var excludes = ids.join(',');
			Main.openPopup('../../admin/users/index.jsp?ID=<%=ID%>&cmd=UpdateContext&mode=selectmultiple&callback=addUserCallback&permission=READ&exclude='+excludes, 'AddUsers', 1050, 700);
		}
		function addUserCallback(userId, name)
		{
			var permissionCode = getPermissionCode();
			var permissionString = getPermissionString(permissionCode);
			var item = Link.getItem('U', userId);
			if (!item) item = new Item('U', userId, name+' ['+permissionString+']', permissionCode, 0);
			Link.addItem(document.forms['share'].members, item);
			membersOnChange();
		}
		function addGroupsOnClick()
		{
			var ids = Link.getListIds(document.forms['share'].members, 'G');
			<%
			if (!sc.hasSystemPermission(Permission.SHARE_TO_EVERYONE))
			{
				%>
				ids[ids.length] = '<%=SystemItems.getId(Group.EVERYONE)%>';
				<%
			}
			%>
			var excludes = ids.join(',');
			Main.openPopup('../../admin/groups/index.jsp?ID=<%=ID%>&cmd=UpdateContext&mode=selectmultiple&callback=addGroupCallback&permission=READ&exclude='+excludes, 'AddGroups', 1050, 700);
		}
		function addGroupCallback(groupId, name)
		{
			var permissionCode = getPermissionCode();
			var permissionString = getPermissionString(permissionCode);
			var item = Link.getItem('G', groupId);
			if (!item) item = new Item('G', groupId, name+' ['+permissionString+']', permissionCode, 0);
			Link.addItem(document.forms['share'].members, item);
			membersOnChange();
		}
		function addProjectsOnClick()
		{
			var ids = Link.getListIds(document.forms['share'].members, 'P');
			var excludes = ids.join(',');
			Main.openPopup('../../my_base/projects/index.jsp?ID=<%=ID%>&cmd=UpdateContext&mode=selectmultiple&callback=addProjectCallback&exclude='+excludes, 'AddProjects', 1050, 700);
		}
		function addProjectCallback(projectId, name)
		{
			var permissionCode = getPermissionCode();
			var permissionString = getPermissionString(permissionCode);
			var item = Link.getItem('P', projectId);
			if (!item) item = new Item('P', projectId, name+' ['+permissionString+']', permissionCode, 0);
			Link.addItem(document.forms['share'].members, item);
			membersOnChange();
		}
		function addPermissionTemplateOnClick()
		{
			var ids = Link.getListIds(document.forms['share'].members, 'T');
			var excludes = ids.join(',');
			Main.openPopup('../../views/permissiontemplates/index.jsp?ID=<%=ID%>&cmd=UpdateContext&mode=selectmultiple&callback=addPermissionTemplateCallback&permission=READ&exclude='+excludes, 'AddPermissionTemplates', 1050, 700);
		}
		function addPermissionTemplateCallback(templateId, name)
		{
			var item = Link.getItem('T', templateId);
			if (!item) item = new Item('T', templateId, name, 0, 0);
			Link.addItem(document.forms['share'].members, item);
			membersOnChange();
		}
		function removeOnClick()
		{
			Link.removeSelected(document.forms['share'].members);
			membersOnChange();
		}
		</script>
	</base:head>
	<base:body onload="<%="initList()"%>">
		<h1><%=headLine%> <base:help helpid="share.setpermissions" /></h1>
		<form name="share" action="submit_share.jsp" onsubmit="return false;">
		<input type="hidden" name="ID" value="<%=ID%>">
		<input type="hidden" name="cmd" value="Save">
		<input type="hidden" name="item_type" value="<%=itemType.name()%>">
		<input type="hidden" name="subcontext" value="<%=subContext%>">
	
		<div class="content filled">
		<table style="width: 100%;">
		<tr>
			<td class="padded"><b>Members</b></td>
			<td class="padded"><b>Permissions</b></td>
			<td></td>
		</tr>
		<tr style="vertical-align: top;">
			<td rowspan="2" style="padding-left: 5px; padding-right: 5px;">
				<select name="members" size="16" multiple onChange="membersOnChange()" style="width: 100%;" 
					<%=!writePermission ? "disabled" : ""%>>
				</select>
				<input type="hidden" name="modifiedUsers" value="">
				<input type="hidden" name="modifiedGroups" value="">
				<input type="hidden" name="modifiedProjects" value="">
				<input type="hidden" name="permissionTemplates" value="">
			</td>
			<td>
				<input type="checkbox" id="read" checked name="read" onClick="permissionsOnClick('read')"><label for="read">Read</label><br>
				<input type="checkbox" id="use" checked name="use" onClick="permissionsOnClick('use')"><label for="use">Use</label><br>
				<input type="checkbox" id="write" name="write" onClick="permissionsOnClick('write')"><label for="write">Write</label><br>
				<input type="checkbox" id="delete" name="delete" onClick="permissionsOnClick('delete')"><label for="delete">Delete</label><br>
				<input type="checkbox" id="setOwner" name="set_owner" onClick="permissionsOnClick('set_owner')"><label for="setOwner">Set owner</label><br>
				<input type="checkbox" id="setPermission" name="set_permission" onClick="permissionsOnClick('set_permission')"><label for="setPermission">Set permission</label><br>
			</td>
			<td>
				<base:buttongroup vertical="true" style="width: 12em;">
					<base:button 
						subclass="leftaligned"
						onclick="addUsersOnClick()" 
						title="Add&nbsp;users&hellip;" 
						tooltip="Add users"
						disabled="<%=!writePermission %>" 
						/>
					<base:button 
						subclass="leftaligned"
						onclick="addGroupsOnClick()" 
						title="Add&nbsp;groups&hellip;"  
						tooltip="Add groups"
						disabled="<%=!writePermission %>" 
					/>
					<base:button 
						subclass="leftaligned"
						onclick="addProjectsOnClick()" 
						title="Add&nbsp;projects&hellip;" 
						tooltip="Add projects"
						disabled="<%=!writePermission %>" 
						/>
					<base:button 
						subclass="leftaligned"
						onclick="addPermissionTemplateOnClick()" 
						title="Templates&hellip;" 
						tooltip="Add permission templates"
						image="add.png"
						disabled="<%=!writePermission %>" 
						/>
					<base:button 
						subclass="leftaligned"
						onclick="removeOnClick()" 
						title="Remove" 
						tooltip="Remove the selected items from this group"
						disabled="<%=!writePermission%>" 
					/>
				</base:buttongroup>
			</td>
		</tr>
		<tr>
			<td colspan="2">
			<%
			if (hasDirectory)
			{
				%>
				<input type="checkbox" name="recursive" value="1" id="recursive">
				<label for="recursive">
				Apply permissions to all subdirectories and their files</label>
				<%
			}
			%>
			</td>
		</tr>
		</table>

			<div class="messagecontainer error" id="share_disabled" style="display:none;">
				You don't have permission to change permissions for the 
				Everyone group. Please contact the server administrator
				if you think you should have that permission.
			</div>
		
		</div>
		</form>
		
		<base:buttongroup subclass="dialogbuttons">
			<base:button onclick="saveSettings();" title="Ok" />
			<base:button onclick="window.close();" title="Cancel" />
		</base:buttongroup>


	</base:body>
	</base:page>
	<%
}
finally
{
	if (users != null) users.close();
	if (groups != null) groups.close();
	if (projects != null) projects.close();
	if (dc != null) dc.close();
}
%>
