<%-- $Id: edit_file_data.jsp 5908 2011-12-13 13:48:46Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.Directory"
	import="net.sf.basedb.core.File"
	import="net.sf.basedb.core.Location"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.BaseException"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="java.util.List"
	import="java.util.Set"
	import="java.util.HashSet"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%
final Item itemType = Item.FILE;
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
try
{
	final File file = File.getById(dc, itemId);
	final String mimeType = file.getMimeType();
	final boolean editable = file.hasPermission(Permission.WRITE) && !file.isWriteProtected() &&
		mimeType != null &&  mimeType.startsWith("text/") && 
		file.getSize() < 102400 && file.getLocation() == Location.PRIMARY;

	if (!editable)
	{
		throw new PermissionDeniedException("Online editing of file is not allowed: " + file);
	}
	final String path = file.getPath().toString();
	final String title = "Edit file -- " + HTML.encodeTags(file.getName());
	%>
	<base:page type="popup" title="<%=title%>">
	<base:head scripts="ajax.js,json2.js" styles="toolbar.css">
		<script language="JavaScript">
		var isModified = false;
		var originalText;
		function init()
		{
			downloadFile();
		}
		function downloadFile()
		{
			var url = 'download/-<%=ID%>-/<%=HTML.javaScriptEncode(path)%>';
			var request = Main.getAjaxRequest();
			request.open("GET", url, true);
			Ajax.setReadyStateHandler(request, onFileLoaded);
			request.send(null);
		}
		function onFileLoaded(request)
		{
			var frm = document.forms['file'];
			frm.filedata.value = request.responseText;
			originalText = frm.filedata.value;
			setModified(false);
			updateLineNumbers();
		}
		function saveFile(closeIfSuccess)
		{
			if (!isModified) return;
			var frm = document.forms['file'];
			var url = 'ajax.jsp?ID=<%=ID%>&cmd=StoreFile&item_id=<%=itemId%>';
			url += '&size='+frm.filedata.value.length;
			var request = Main.getAjaxRequest();
			request.closeIfSuccess = closeIfSuccess;
			request.open("POST", url, true);
			Ajax.setReadyStateHandler(request, onFileStored);
			request.send(frm.filedata.value);
		}
		function onFileStored(request)
		{
			var response = Ajax.parseJsonResponse(request.responseText);
			alert(response.message);
			if (response.status != 'ok')
			{
				return;
			}
			setModified(false);
			if (request.closeIfSuccess) 
			{
				closeWindow(true);
			}
		}
		function closeFile(autoSave)
		{
			if (isModified)
			{
				if (autoSave || confirm('The file has been modified. Save changes?'))
				{
					saveFile(true);
					return;
				}
			}
			closeWindow(false);
		}
		function closeWindow(refreshParent)
		{
			if (refreshParent)
			{
				location.href = getRoot() + 'common/close_popup.jsp?refresh_opener=1&wait=0';
			}
			else
			{
				window.close();
			}
		}
		
		function detectChanges()
		{
			if (!isModified)
			{
				var frm = document.forms['file'];
				if (frm.filedata.value != originalText) setModified(true);
			}
		}
		function setModified(modified)
		{
			isModified = modified;
		}
		function detectTab(event)
		{
			var key = Main.getEventKeyCode(event);
			if (key == 9)
			{
				var frm = document.forms['file'];
				replaceSelection(frm.filedata, '\t');
				return false;
			}
			return true;
		}

		function updateLineNumbers()
		{
			var frm = document.forms['file'];
			var totalScroll = frm.filedata.scrollHeight;
			var topScroll = frm.filedata.scrollTop;
			var textHeight = frm.filedata.offsetHeight;
			var topLine = 1;
			var scrollRemain = 0;
			if (totalScroll > textHeight || topScroll > 0)
			{
				var numLines = frm.filedata.value.split('\n').length;
				var scrollPerLine = totalScroll / numLines;
				topLine = 1 + Math.floor(topScroll / scrollPerLine);
				scrollRemain = topScroll - scrollPerLine * (topLine - 1);
			}
			var lineNumberHTML = '';
			for (var i = topLine; i < topLine + 100; i++)
			{
				lineNumberHTML += i + '<br>';
			}
			var lineNumberElem = document.getElementById('linenumbers');
			lineNumberElem.innerHTML = lineNumberHTML;
			lineNumberElem.scrollTop = scrollRemain;
		}
		
		/*
			Replace the selected text with the given text. Code based on
			Trac: http://projects.edgewall.com/trac/
		*/
		function replaceSelection(field, text)
		{
			field.focus();
		    var start, end, sel, scrollPos;
		    if (typeof(document["selection"]) != "undefined") 
		    {
		      sel = document.selection.createRange().text;
		    } 
		    else if (typeof(field["setSelectionRange"]) != "undefined") 
		    {
		      start = field.selectionStart;
		      end = field.selectionEnd;
		      scrollPos = field.scrollTop;
		      sel = field.value.substring(start, end);
		    }
		    if (sel.match(/ $/)) 
		    { 
		    	// exclude ending space char, if any
		      	sel = sel.substring(0, sel.length - 1);
		    }
		    if (typeof(document["selection"]) != "undefined") 
		    {
		      var range = document.selection.createRange().text = text;
		    } 
		    else if (typeof(field["setSelectionRange"]) != "undefined") 
		    {
		    	field.value = field.value.substring(0, start) + text +
		      		field.value.substring(end);
		      field.setSelectionRange(start + text.length, start + text.length);
		      field.scrollTop = scrollPos;
		    }
		}
		</script>
		<style>
		#linenumbers
		{
			position: absolute;
			top: 0em;
			left: 0em;
			bottom: 0em;
			width: 3.6em;
			padding-right: 0.4em;
			text-align: right;
			overflow: hidden;
			color: #224488;
			background: #E8E8E8;
			font-family: monospace;
			border-right: 2px solid #224488;
		}
		
		#filecontainer
		{
			position: absolute;
			top: 0em;
			left: 4em;
			bottom: 0em;
			right: 0em;
			margin-left: 2px;
		}
		
		#filedata
		{
			width: 100%; 
			height: 100%; 
			border: 0px;
			margin: 0px;
			padding: 0px;
		}
		</style>
	</base:head>
	<base:body onload="init()">
		<h1><%=title%> <base:help helpid="file.edit.data" /></h1>
		
		<form action="index.jsp?ID=<%=ID%>" method="post" name="file" onsubmit="return false;">
		<input type="hidden" name="cmd" value="UpdateItem">
		
		<div class="content bottomborder" style="overflow: hidden;">
			<div id="linenumbers"></div>
			<div id="filecontainer">
			<textarea id="filedata" name="filedata" 
				wrap="off"
				onkeydown="return detectTab(event);"
				onkeyup="detectChanges()"
				onscroll="updateLineNumbers()"
				>Loading file; please wait...</textarea>
			</div>
		</div>
		</form>
		
		<base:buttongroup subclass="dialogbuttons">
			<base:button onclick="closeFile(true)" title="Save" />
			<base:button onclick="closeFile(false)" title="Close" />
		</base:buttongroup>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
