<%-- $Id $
	------------------------------------------------------------------
	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
--%>
<%@ page pageEncoding="UTF-8" session="false" contentType="application/json" 
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.PhysicalBioAssay"
	import="net.sf.basedb.core.Extract"
	import="net.sf.basedb.core.BioMaterial"
	import="net.sf.basedb.core.BioMaterialEventSource"
	import="net.sf.basedb.core.SpecialQuery"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.Type"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.query.Restrictions"
	import="net.sf.basedb.core.query.Expressions"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.biomaterial.ParentsTransformer"
	import="net.sf.basedb.util.error.ThrowableUtil"
	import="net.sf.basedb.clients.web.WebException"
	import="net.sf.basedb.clients.web.util.HTML"
	import="org.json.simple.JSONObject"
	import="org.json.simple.JSONArray"
	import="java.util.Date"
	import="java.util.Arrays"
	import="java.util.List"
	import="java.util.Set"
	import="java.util.HashSet"
%>
<%
response.setHeader("Cache-Control", "no-cache, max-age=0");
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final String cmd = request.getParameter("cmd");
final String root = request.getContextPath()+"/";
final int itemId = Values.getInt(request.getParameter("item_id"));
out.clear();
JSONObject json = new JSONObject();
json.put("status", "ok");
DbControl dc = null;
try
{
	if ("GetSources".equals(cmd))
	{
		dc = sc.newDbControl();
		PhysicalBioAssay pba = PhysicalBioAssay.getById(dc, itemId);
		json.put("id", pba.getId());
		json.put("name", pba.getName());
		
		SpecialQuery<BioMaterialEventSource> sourceQuery = pba.getCreationEvent().getEventSources();
		sourceQuery.order(Orders.asc(Hql.property("position")));
		sourceQuery.order(Orders.asc(Hql.property("bioMaterial.name")));
		JSONArray jsonSources = new JSONArray();
		for (BioMaterialEventSource src : sourceQuery.list(dc))
		{
			JSONObject jsonSrc = new JSONObject();
			try
			{
				BioMaterial extract = src.getBioMaterial();
				jsonSrc.put("id", extract.getId());
				jsonSrc.put("name", extract.getName());
				jsonSrc.put("position", src.getPosition());
				jsonSources.add(jsonSrc);
			}
			catch (PermissionDeniedException ex)
			{}
		}
		json.put("sources", jsonSources);
		
		dc.commit();
	}
	else if ("GetAllExtractIds".equals(cmd))
	{
		dc = sc.newDbControl();
		
		Integer[] bioAssayIds = Values.getInt(request.getParameter("bioassay_ids").split(","));

		// Load all extracts that are directly linked with the given physical bioassays
		ItemQuery<Extract> query = Extract.getQuery();
		query.include(Include.ALL);
		query.join(Hql.innerJoin("childCreationEvents", "cce"));
		query.join(Hql.innerJoin("cce", "event", "evt"));
		query.restrict(Restrictions.in(Hql.property("evt", "physicalBioAssay"), Expressions.parameter("bioAssays")));
		query.setParameter("bioAssays", Arrays.asList(bioAssayIds), Type.INT);
		List<Extract> primaryExtracts = query.list(dc);
		
		// Use the ParentsTransformer to load all parent extracts as well
		Set<Extract> allExtracts = new HashSet<Extract>();
		new ParentsTransformer(dc, true, Extract.getQuery()).transform(primaryExtracts, allExtracts);
		
		JSONArray jsonExtracts = new JSONArray();
		for (Extract e : allExtracts)
		{
			jsonExtracts.add(e.getId());
		}
		
		json.put("allExtractIds", jsonExtracts);
		dc.commit();
	}
	else
	{
		throw new WebException("popup", "Invalid command", "The command {1} is not recognised as a valid command.", cmd);
	}
}
catch (Throwable t)
{
	t.printStackTrace();
	json.clear();
	json.put("status", "error");
	json.put("message", t.getMessage());
	json.put("stacktrace", ThrowableUtil.stackTraceToString(t));
}
finally
{
	json.writeJSONString(out);
	out.flush();
	if (dc != null) dc.close();
}
%>